@@ -9,7 +9,7 @@
 #include "numpy/arrayobject.h"
 
 #include "npy_config.h"
-#include "npy_pycompat.h"
+
 #include "npy_import.h"
 
 #include "common.h"
@@ -25,6 +25,8 @@
 #include "alloc.h"
 #include "npy_buffer.h"
 #include "shape.h"
+#include "multiarraymodule.h"
+#include "array_api_standard.h"
 
 /*******************  array attribute get and set routines ******************/
 
@@ -385,16 +387,16 @@ array_descr_set(PyArrayObject *self, PyObject *arg, void *NPY_UNUSED(ignored))
 
     /* check that we are not reinterpreting memory containing Objects. */
     if (_may_have_objects(PyArray_DESCR(self)) || _may_have_objects(newtype)) {
-        static PyObject *checkfunc = NULL;
         PyObject *safe;
 
-        npy_cache_import("numpy._core._internal", "_view_is_safe", &checkfunc);
-        if (checkfunc == NULL) {
+        if (npy_cache_import_runtime(
+                "numpy._core._internal", "_view_is_safe",
+                &npy_runtime_imports._view_is_safe) == -1) {
             goto fail;
         }
 
-        safe = PyObject_CallFunction(checkfunc, "OO",
-                                     PyArray_DESCR(self), newtype);
+        safe = PyObject_CallFunction(npy_runtime_imports._view_is_safe,
+                                     "OO", PyArray_DESCR(self), newtype);
         if (safe == NULL) {
             goto fail;
         }
@@ -883,12 +885,6 @@ array_itemset(PyArrayObject *self, PyObject *args)
     return NULL;
 }
 
-static PyObject *
-array_device(PyArrayObject *self, void *NPY_UNUSED(ignored))
-{
-    return PyUnicode_FromString("cpu");
-}
-
 NPY_NO_EXPORT PyGetSetDef array_getsetlist[] = {
     {"ndim",
         (getter)array_ndim_get,
